﻿Type=Class
Version=5.2
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Class module v1.02 (Wrapped by: T201016)
Sub Class_Globals
	Type MyType1 (SavedFilePath, SavedFileName As String, iTimeT, iTimeZ As String, pattern As Char)
	Dim Globals_Type As MyType1
	Type MyType2 (maxlat, maxlon As String, minlat, minlon As String)
	Dim Globals_Bounds As MyType2
	
	Dim Rpe_RoutePointExtension As List
End Sub

'Splits the given text around matches of the pattern (default pattern: "∆")
Public Sub Initializes(pattern As String)
	If Not(Globals_Type.IsInitialized) Then
		Globals_Type.Initialize
	End If
	Globals_Type.pattern = pattern
End Sub

'Removes all the items from the list.
'Initializes the fields to their default value.
Public Sub ClearAll
	Globals_Type.Initialize
	If Rpe_RoutePointExtension.IsInitialized Then Rpe_RoutePointExtension.Clear
End Sub

'Initializes in «metadata» the bounds to their default value.
Public Sub SetBounds(maxlat As String, maxlon As String, minlat As String, minlon As String)
	If Not(Globals_Bounds.IsInitialized) Then
		Globals_Bounds.Initialize
	End If
	Globals_Bounds.maxlat = maxlat
	Globals_Bounds.maxlon = maxlon
	Globals_Bounds.minlat = minlat
	Globals_Bounds.minlon = minlon
End Sub

'Initializes an empty location object. You can add parameters to this method if needed.
'path: specified Dir.
'filename: specified FileName. See: (Initialize the Globals_Type.SavedFileName).
'Fix_altitude: initialize the fix altitude (meters).
'Fix_accuracy: initialize the fix accuracy (meters).
'Fix_wpts: initialize the fix waypoint's (segment WPT).
'Fix_time: initialize the fix time (HH:mm:ss).
'Fix_trkname: initialize the specified track name.
'Globals_Gpxx_RoutePointExtension: initialize the whole specified segment (RoutePointExtension).
'Chr_pattern: splits the given text around matches of the pattern (default: "∆").
Public Sub Initialize(path As String, Fix_trkname As String, Fix_time As List, Fix_accuracy As List, Fix_altitude As List, Fix_wpts As List, Chr_pattern As String) As Boolean
	Private i As Int

	Fix_trkname = ""
	Fix_time.Initialize
	Fix_accuracy.Initialize
	Fix_altitude.Initialize
	Fix_wpts.Initialize
	Rpe_RoutePointExtension.Initialize

	SetBounds("0.0000000","0.0000000","0.0000000","0.0000000") 'see: <bounds maxlat="51.0" maxlon="7.0" minlat="51.0" minlon="7.0"/>

	Globals_Type.Initialize
	Globals_Type.SavedFilePath = ""
	Globals_Type.SavedFileName = ""
	Globals_Type.iTimeT = ""
	Globals_Type.iTimeZ = ""
	Globals_Type.pattern = ""
	
	If Globals_Type.pattern = "" Then Initializes("∆")

	If path.Length = 0 Then Globals_Type.SavedFilePath = File.DirRootExternal Else Globals_Type.SavedFilePath = path
	
	If Chr_pattern <> "∆" Then Globals_Type.pattern = Chr_pattern
	
	If Fix_accuracy.IsInitialized Then
		DateTime.TimeFormat = "HH:mm:ss"
		DateTime.DateFormat = "yyyy-MM-dd"
		Globals_Type.iTimeT = DateTime.Date(DateTime.Now)
		Globals_Type.iTimeZ = DateTime.Time(DateTime.Now)
		Globals_Type.iTimeZ = Globals_Type.iTimeZ.Replace(":","")
		Dim filename As String = "RTE " & Globals_Type.iTimeT & "_" & Globals_Type.iTimeZ & ".GPX"
		i = 0
		Do While File.Exists(path, filename)
			i = i +1
			filename = "RTE " & Globals_Type.iTimeT & "_" & Globals_Type.iTimeZ & "~" & i & ".GPX"
		Loop
		
		Globals_Type.SavedFileName = filename
		Return True
	Else
		Globals_Type.SavedFileName = ""
		Return False
	End If
End Sub

'Save to a file trail GPS which the move.
'path: specified Dir.
'filename: specified FileName. See: (Initialize the Globals_Type.SavedFileName).
'Fix_trkname: initialize the specified track name.
'Fix_trkcmt: initialize the specified track comment.
'DisplayColor: initialize the specified track color (ex. "Blue").
'Fix_time: initialize the fix time (HH:mm:ss).
'Fix_wpts: initialize the fix waypoint's (segment WPT).
'Fix_accuracy: initialize the fix accuracy (meters).
'Fix_altitude: initialize the fix altitude (meters).
'html_schemas: (ex.: "http://www.garmin.com/xmlschemas/GpxExtensions/v3").
Public Sub CreateFileTracks(path As String, filename As String, Fix_trkname As String, DisplayColor As String, Fix_trkcmt As String, Fix_time As List, Fix_accuracy As List, Fix_altitude As List, Fix_wpts As List, html_schemas As String) As Boolean
 	Dim TextWriter1 As TextWriter
	Dim line As String

	DateTime.TimeFormat = "HH:mm:ss"
	DateTime.DateFormat = "yyyy-MM-dd"
	Globals_Type.iTimeT = DateTime.Date(DateTime.Now)
	Globals_Type.iTimeZ = DateTime.Time(DateTime.Now)

	Try
		line = ""
		TextWriter1.Initialize(File.OpenOutput(path, filename, False))
		
		line = "<?xml version='1.0' encoding='UTF-8' standalone='no' ?>"&CRLF
		line = line & "<gpx xmlns='http://www.topografix.com/GPX/1/1' creator='MapSource 6.16.3' version='1.1' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xsi:schemaLocation='http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/Globals_Type.xsd'>"&CRLF&CRLF
		
		line = line & "  <metadata>"&CRLF
		line = line & "    <link href='http://www.garmin.com'>"&CRLF
		line = line & "      <text>Garmin International</text>"&CRLF
		line = line & "    </link>"&CRLF
		line = line & "    <time>"&Globals_Type.iTimeT&"T"&Globals_Type.iTimeZ&"Z</time>"&CRLF
		line = line & "    <bounds maxlat='"&Globals_Bounds.maxlat&"' maxlon='"&Globals_Bounds.maxlon&"' minlat='"&Globals_Bounds.minlat&"' minlon='"&Globals_Bounds.minlon&"'/>"&CRLF
		line = line & "  </metadata>"&CRLF&CRLF
		
		line = line & wpt(Fix_wpts)

		line = line & "  <trk>"&CRLF
		line = line & "    <name>"&Fix_trkname&"</name>"&CRLF
		line = line & "    <cmt>"&Fix_trkcmt&"</cmt>"&CRLF
		line = line & "    <extensions>"&CRLF
		line = line & "      <gpxx:TrackExtension xmlns:gpxx='"&html_schemas&"'>"&CRLF
		line = line & "        <gpxx:DisplayColor>"&DisplayColor&"</gpxx:DisplayColor>"&CRLF
		line = line & "      </gpxx:TrackExtension>"&CRLF
		line = line & "    </extensions>"&CRLF
		line = line & "    <trkseg>"&CRLF

		line = line & trkpt(Fix_accuracy, Fix_time, Fix_altitude)

		line = line & "    </trkseg>"&CRLF
		line = line & "  </trk>"&CRLF&CRLF

		line = line & "</gpx>"
		line = line.Replace("'",Chr(34))
		TextWriter1.WriteLine(line)
		TextWriter1.Close :	line = ""
		Return True
	Catch
		line = ""
		If TextWriter1.IsInitialized Then TextWriter1.Close
		Return False
	End Try
End Sub

'Save to a file routes GPS which the move.
'path: specified Dir.
'filename: specified FileName. See: (Initialize the Globals_Type.SavedFileName).
'Fix_trkname: initialize the specified route name.
'DisplayColor: initialize the specified track color (ex. "Blue").
'Fix_trkcmt: initialize the specified route comment.
'Fix_time: initialize the fix time (HH:mm:ss).
'Fix_wpts: initialize the fix waypoint's (segment WPT).
'Fix_rtes: initialize the fix rtepoint's (segment RTE).
'Fix_accuracy: initialize the fix accuracy (meters).
'Fix_altitude: initialize the fix altitude (meters).
'html_schemas: (ex.: "http://www.garmin.com/xmlschemas/GpxExtensions/v3").
Public Sub CreateFileRoutes(path As String, filename As String, Fix_trkname As String, DisplayColor As String, Fix_trkcmt As String,  Fix_time As List, Fix_accuracy As List, Fix_altitude As List, Fix_wpts As List, Fix_rtes As List, html_schemas As String) As Boolean
 	Dim TextWriter1 As TextWriter
	Dim line As String

	DateTime.TimeFormat = "HH:mm:ss"
	DateTime.DateFormat = "yyyy-MM-dd"
	Globals_Type.iTimeT = DateTime.Date(DateTime.Now)
	Globals_Type.iTimeZ = DateTime.Time(DateTime.Now)

	Try
		line = ""
		TextWriter1.Initialize(File.OpenOutput(path, filename, False))

		line = "<?xml version='1.0' encoding='UTF-8' standalone='no' ?>"&CRLF
		line = line & "<gpx xmlns='http://www.topografix.com/GPX/1/1' creator='MapSource 6.16.3' version='1.1' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xsi:schemaLocation='http://www.garmin.com/xmlschemas/GpxExtensions/v3 http://www.garmin.com/xmlschemas/GpxExtensions/v3/GpxExtensionsv3.xsd http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd'>"&CRLF&CRLF
		
		line = line & "  <metadata>"&CRLF
		line = line & "    <link href='http://www.garmin.com'>"&CRLF
		line = line & "      <text>Garmin International</text>"&CRLF
		line = line & "    </link>"&CRLF
		line = line & "    <time>"&Globals_Type.iTimeT&"T"&Globals_Type.iTimeZ&"Z</time>"&CRLF
		line = line & "    <bounds maxlat='"&Globals_Bounds.maxlat&"' maxlon='"&Globals_Bounds.maxlon&"' minlat='"&Globals_Bounds.minlat&"' minlon='"&Globals_Bounds.minlon&"'/>"&CRLF
		line = line & "  </metadata>"&CRLF&CRLF

		line = line & wpt(Fix_wpts)
		
		line = line & rte(Fix_rtes, html_schemas)

		line = line & "  <trk>"&CRLF
		line = line & "    <name>"&Fix_trkname&"</name>"&CRLF
		line = line & "    <cmt>"&Fix_trkcmt&"</cmt>"&CRLF
		line = line & "    <extensions>"&CRLF
		line = line & "      <gpxx:TrackExtension xmlns:gpxx='"&html_schemas&"'>"&CRLF
		line = line & "        <gpxx:DisplayColor>"&DisplayColor&"</gpxx:DisplayColor>"&CRLF
		line = line & "      </gpxx:TrackExtension>"&CRLF
		line = line & "    </extensions>"&CRLF
		line = line & "    <trkseg>"&CRLF
		
		line = line & trkpt(Fix_accuracy, Fix_time, Fix_altitude)

		line = line & "    </trkseg>"&CRLF
		line = line & "  </trk>"&CRLF&CRLF

		line = line & "</gpx>"&CRLF
		line = line.Replace("'",Chr(34))
		TextWriter1.WriteLine(line)
		TextWriter1.Close :	line = ""
		Return True
	Catch
		line = ""
		If TextWriter1.IsInitialized Then TextWriter1.Close
		Return False
	End Try
End Sub

'Fill out the segment WPT.
'fix_wpt: lat,lon,time,name,cmt,desc,sym,WaypointExtension,DisplayMode.
Sub wpt(fix_wpt As List) As String
	Private linewpt As String = ""
	Private words() As String
	
	DateTime.TimeFormat = "HH:mm:ss"
	DateTime.DateFormat = "yyyy-MM-dd"

	Try
		Dim str As StringFunctionGPX
		For i = 0 To fix_wpt.Size - 1
			words = Regex.Split(Globals_Type.pattern, fix_wpt.Get(i))
			Globals_Type.iTimeT = str.SplitGetWord(words(2), "", "", " ",1)
			Globals_Type.iTimeZ = str.SplitGetWord(words(2), "", "", " ",2)
			linewpt = linewpt & "  <wpt lat='"&words(0)&"' lon='"&words(1)&"'>"&CRLF
			linewpt = linewpt & "    <time>"&Globals_Type.iTimeT&"T"&Globals_Type.iTimeZ&"Z</time>"&CRLF
			linewpt = linewpt & "    <name>"&words(3)&"</name>"&CRLF
			linewpt = linewpt & "    <cmt>"&words(4)&"</cmt>"&CRLF
			linewpt = linewpt & "    <desc>"&words(5)&"</desc>"&CRLF
			linewpt = linewpt & "    <sym>"&words(6)&"</sym>"&CRLF
			linewpt = linewpt & "    <extensions>"&CRLF
			linewpt = linewpt & "      <gpxx:WaypointExtension xmlns:gpxx='"&words(7)&"'>"&CRLF
			linewpt = linewpt & "        <gpxx:DisplayMode>"&words(8)&"</gpxx:DisplayMode>"&CRLF
			linewpt = linewpt & "      </gpxx:WaypointExtension>"&CRLF
			linewpt = linewpt & "    </extensions>"&CRLF
			linewpt = linewpt & "  </wpt>"&CRLF&CRLF
		Next
	Catch
		Return ""
	End Try
	Return linewpt
End Sub

'Fill out the segment TRKPT.
'fix_trkpt: GPS coordinates of the points (coordinate format 00.000000).
'time_trkpt: GPS fix time (HH:mm:ss).
'altitude_trkpt: GPS fix altitude (meters).
Sub trkpt(fix_trkpt As List, time_trkpt As List, altitude_trkpt As List) As String
	Private linetrkpt As String = ""
	Private words() As String
	
	DateTime.TimeFormat = "HH:mm:ss"
	DateTime.DateFormat = "yyyy-MM-dd"

	Try
		For i = 0 To fix_trkpt.Size - 1
			words = Regex.Split(Globals_Type.pattern, time_trkpt.Get(i))
			Globals_Type.iTimeT = words(0)
			Globals_Type.iTimeZ = words(1)
			words = Regex.Split(Globals_Type.pattern, fix_trkpt.Get(i))
			linetrkpt = linetrkpt & "      <trkpt lat='"&words(0)&"' lon='"&words(1)&"'>"&CRLF
			words = Regex.Split(Globals_Type.pattern, altitude_trkpt.Get(i))
			linetrkpt = linetrkpt & "        <ele>"&words(1)&"</ele>"&CRLF
			linetrkpt = linetrkpt & "        <time>"&Globals_Type.iTimeT&"T"&Globals_Type.iTimeZ&"Z</time>"&CRLF
			linetrkpt = linetrkpt & "      </trkpt>"&CRLF
		Next
	Catch
		Return ""
	End Try
	Return linetrkpt
End Sub

'Fill out the segment RTE.
'fix_rte: rte_name,rte_cmt,rte_IsAutoNamed,
'fix_rte: rtept_lat,rtept_lon,rtept_time,rtept_name,rtept_cmt,rtept_desc,rtept_sym.
Sub rte(Fix_rte As List, html_schemas As String) As String
	Private linerte As String = ""
	Private words() As String
	
	DateTime.TimeFormat = "HH:mm:ss"
	DateTime.DateFormat = "yyyy-MM-dd"

	Try
		Dim str As StringFunctionGPX
		For i = 0 To Fix_rte.Size - 1
			words = Regex.Split(Globals_Type.pattern, Fix_rte.Get(i))
			Globals_Type.iTimeT = str.SplitGetWord(words(5), "", "", " ",1)
			Globals_Type.iTimeZ = str.SplitGetWord(words(5), "", "", " ",2)
			linerte = linerte & "  <rte>"&CRLF
			linerte = linerte & "    <name>"&words(0)&"</name>"&CRLF
			linerte = linerte & "    <cmt>"&words(1)&"</cmt>"&CRLF
			linerte = linerte & "    <extensions>"&CRLF
			linerte = linerte & "      <trp:ShapingPoint></trp:ShapingPoint>"&CRLF
			linerte = linerte & "      <gpxx:RouteExtension xmlns:gpxx='"&html_schemas&"'>"&CRLF
			linerte = linerte & "        <gpxx:IsAutoNamed>"&words(2)&"</gpxx:IsAutoNamed>"&CRLF
			linerte = linerte & "      </gpxx:RouteExtension>"&CRLF
			linerte = linerte & "    </extensions>"&CRLF
			linerte = linerte & "    <rtept lat='"&words(3)&"' lon='"&words(4)&"'>"&CRLF
			linerte = linerte & "      <time>"&Globals_Type.iTimeT&"T"&Globals_Type.iTimeZ&"Z</time>"&CRLF
			linerte = linerte & "      <name>"&words(6)&"</name>"&CRLF
			linerte = linerte & "      <cmt>"&words(7)&"</cmt>"&CRLF
			linerte = linerte & "      <desc>"&words(8)&"</desc>"&CRLF
			linerte = linerte & "      <sym>"&words(9)&"</sym>"&CRLF
			linerte = linerte & "      <extensions>"&CRLF

			linerte = linerte & RoutePointExtension(html_schemas)

			linerte = linerte & "      </extensions>"&CRLF
			linerte = linerte & "    </rtept>"&CRLF
			linerte = linerte & "  </rte>"&CRLF&CRLF
		Next
	Catch
		Return ""
	End Try
	Return linerte
End Sub

'Example:
'gpxx_rpt.Initialize
'gpxx_rpt.Put("lat","51.8660035") ... [ContainsKey = "lat"]
'gpxx_rpt.Put("lat","48.8660035") ... [ContainsKey = "lon"]
'gpxx_rpt(gpxx_rpt)
'In this example the function will Return: «gpxx:rpt lat="51.8660035" lon="48.8660035"/»
Public Sub gpxx_rpt(gpxx_rpts As Map) As String
	Private linerpt As String = ""

	Try
		If Not(Rpe_RoutePointExtension.IsInitialized) Then Rpe_RoutePointExtension.Initialize
		If gpxx_rpts.ContainsKey("lat") And gpxx_rpts.ContainsKey("lon") Then
			linerpt = "          <gpxx:rpt lat='"&gpxx_rpts.Get("lat")&"' lon='"&gpxx_rpts.Get("lon")&"'/>"&CRLF
			Rpe_RoutePointExtension.Add(linerpt)
		End If
	Catch
		Return ""
	End Try
	Return linerpt
End Sub

'Example:
'gpxx_rpt.Initialize
'gpxx_rpt.Put("lat","51.8660035") ... [ContainsKey = "lat"]
'gpxx_rpt.Put("lat","48.8660035") ... [ContainsKey = "lon"]
'gpxx_rpt.Put("sub","06000f544d") ... [ContainsKey = "sub"]
'gpxx_rpt(gpxx_rpt)
'In this example the function will Return:
'«gpxx:rpt lat="51.8660035" lon="48.8660035"»
'  «gpxx:Subclass»06000f544d«/gpxx:Subclass»
'«/gpxx:rpt»
Public Sub gpxx_rptsubclass(gpxx_rpts As Map) As String
	Private linerpt As String = ""
	Try
		If Not(Rpe_RoutePointExtension.IsInitialized) Then Rpe_RoutePointExtension.Initialize
		If gpxx_rpts.ContainsKey("lat") And gpxx_rpts.ContainsKey("lon") And gpxx_rpts.ContainsKey("sub") Then
			linerpt = linerpt & "          <gpxx:rpt lat='"&gpxx_rpts.Get("lat")&"' lon='"&gpxx_rpts.Get("lon")&"'>"&CRLF
			linerpt = linerpt & "            <gpxx:Subclass>"&gpxx_rpts.Get("sub")&"</gpxx:Subclass>"&CRLF
			linerpt = linerpt & "          </gpxx:rpt>"&CRLF
			Rpe_RoutePointExtension.Add(linerpt)
		End If
	Catch
		Return ""
	End Try
	Return linerpt
End Sub

'Example:
'gpxx_Subclass("000000000000")
'In this example the function will Return: «gpxx:Subclass»000000000000«/gpxx:Subclass»".
Public Sub gpxx_subclass(opt_subclass As String) As String
	Private linerpt As String

	Try
		If Not(Rpe_RoutePointExtension.IsInitialized) Then Rpe_RoutePointExtension.Initialize
		If opt_subclass.Length > 0 Then
			linerpt = "          <gpxx:Subclass>"&opt_subclass&"</gpxx:Subclass>"&CRLF
			Rpe_RoutePointExtension.Add(linerpt)
		End If
	Catch
		Return ""
	End Try
	Return linerpt
End Sub

'Fill out the segment RoutePointExtension.
'Rpe_RoutePointExtension: initialize elements segment (gpxx:RoutePointExtension).
'xml_schemas: initialize: -  "http://www.garmin.com/xmlschemas/GpxExtensions/v3".
Sub RoutePointExtension(xml_schemas As String) As String
	Private linerpe As String = ""
	Private http As String = "http://www.garmin.com/xmlschemas/GpxExtensions/v3"

	If Not(Rpe_RoutePointExtension.IsInitialized) Then
		Return ""
	Else
		If xml_schemas.Length > 0 Then http = xml_schemas
	End If

	Try
		If Rpe_RoutePointExtension.Size > 0 Then
			For i = 0 To Rpe_RoutePointExtension.Size - 1
				linerpe = linerpe & "        <gpxx:RoutePointExtension xmlns:gpxx='"&http&"'>"&CRLF
				linerpe = linerpe & Rpe_RoutePointExtension.Get(i)
				linerpe = linerpe & "        </gpxx:RoutePointExtension>"&CRLF
			Next
		End If
	Catch
		Return ""
	End Try
	Return linerpe
End Sub