﻿Type=Class
Version=5.2
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Class module v1.02 (Wrapped by: T201016)
Sub Class_Globals
	Dim gpx As String = ""
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initializes
	gpx = ""
End Sub

'Reads the entire file end returns its text.
'The files is assumed to be encoded with UTF8.
Public Sub GetText(path As String, filename As String) As String
	Initializes
	If File.Exists(path, filename) Then
		gpx = File.GetText(path, filename)
	End If
	Return gpx
End Sub

'Used to extract text from a string between to other blocks of text.
'Example:
'MidExtract("«time»2012-01-27T10:07:32Z«/time»", "T", "Z")
'In this example the function will Return: "10:07:32".
Public Sub MidExtract(paramString1 As String, paramString2 As String, paramString3 As String) As String
	Private str As String = ""

	If (LenString(paramString1) = 0) Then Return ""
	If paramString2.Length = 0 And paramString3.Length = 0 Then	Return paramString1
	If ((paramString1.Contains(paramString2)) And (paramString1.Contains(paramString3))) Then

		str = paramString1.Substring2(paramString1.IndexOf(paramString2), paramString1.indexOf(paramString3))
		str = str.Substring2(paramString2.Length, str.Length).Trim
       
		Return str
	Else
   		Return ""
	End If
End Sub

'Returns the length of this string.
Public Sub LenString(paramString1 As String) As Int
	Return paramString1.Length
End Sub

'Returns just the one element selected with GetElement from the string. 
'Example:
'SplitGetWord("This is a current string.", "", "", " ", 2)
'SplitGetWord("«T»This is a current string«Z»", "«T»", "«Z»", " ", 2)
'Splits the given text around matches of the pattern.
'In this example the function will Return: "is"
Public Sub SplitGetWord(CurrentString As String, paramString1 As String, paramString2 As String, Pattern As String, GetElement As Int) As String
	Dim LocalList As List
	Dim words() As String, str As String = ""
	
	If Pattern.Length = 0 Then Pattern = "∆"
	
	If CurrentString.Length = 0 Then Return str
	CurrentString = MidExtract(CurrentString, paramString1, paramString2)

	LocalList.Initialize
	words = Regex.Split(Pattern, CurrentString)
	If words.Length > 0 Then
		For i = 0 To words.Length - 1
			LocalList.Add(words(i))
		Next
		str = iif((GetElement - 1 <= LocalList.Size - 1), LocalList.Get(GetElement - 1), "")
	End If
	Return str
End Sub

'This Is a single line If conditional test.  If Value1 Is True, Value2 Is returned, otherwise
'Value3 will be returned. Works with strings and booleans.
'Example:
'MyBoolean = iif(FirstTime = True, StartMyService = True, StartMyService = False)
'In this example the function will Return: True or False
Sub iif(paramString1 As Boolean, paramString2 As String, paramString3 As String) As String
	If paramString1 Then
		Return paramString2
	Else
		Return paramString3
	End If
End Sub

'Ex:
'«metadata»
'  «link href="http://www.garmin.com"»
'«/metadata»
'Returns the "http://www.garmin.com" of string.
Public Sub Get_link_metadata As String
	If gpx.Length = 0 Then Return ""

	Dim str As String = MidExtract(gpx, "<metadata>","</metadata>")
	str = MidExtract(str, "<link href="&Chr(34), Chr(34)&">")
	Return str
End Sub
'Ex:
'«metadata»
'  «text»Garmin International«/text»
'«/metadata»
'Returns the "Garmin International" of string.
Public Sub Get_text_metadata As String
	If gpx.Length = 0 Then Return ""
	
	Dim str As String = MidExtract(gpx, "<metadata>","</metadata>")
	str = MidExtract(str, "<text>", "</text>")
	Return str
End Sub
'Ex:
'«metadata»
'  «time»2014-11-03T17:51:45Z«/time»
'«/metadata»
'Returns the "2014-11-03 17:51:45" of string.
Public Sub Get_time_metadata As String
	If gpx.Length = 0 Then Return ""
	
	Dim str As String = MidExtract(gpx, "<metadata>","</metadata>")
	str = MidExtract(str, "<time>", "T") &" "& MidExtract(str, "T", "Z")
	Return str
End Sub
'Ex:
'«metadata»
'  «bounds maxlat="52.715650834143162" ... minlon="7.670284537598491"/»
'«/metadata»
'Returns the Key:maxlat,Value:52.715650834143162,...,... of string in Map.
'Pattern: splits the given text around matches of the pattern (default: "∆").
Public Sub Get_bounds_metadata(Pattern As String) As Map
	Dim bounds As Map
	bounds.Initialize
	
	If Pattern.Length = 0 Then Pattern = "∆"
	
	If gpx.Length > 0 Then 
		Dim str As String = MidExtract(gpx, "<bounds","/>")
		Dim maxlat,maxlon,minlat,minlon As String
		maxlat = SplitGetWord(str, "",""," ",1)
		maxlon = SplitGetWord(str, "",""," ",2)
		minlat = SplitGetWord(str, "",""," ",3)
		minlon = SplitGetWord(str, "",""," ",4)
		bounds.Put("maxlat", maxlat.Replace(Chr(34),"").Replace("maxlat","").Replace("=","").Replace(Chr(34),""))
		bounds.Put("maxlon", maxlon.Replace(Chr(34),"").Replace("maxlon","").Replace("=","").Replace(Chr(34),""))
		bounds.Put("minlat", minlat.Replace(Chr(34),"").Replace("minlat","").Replace("=","").Replace(Chr(34),""))
		bounds.Put("minlon", minlon.Replace(Chr(34),"").Replace("minlon","").Replace("=","").Replace(Chr(34),""))
	End If
	Return bounds
End Sub
'Ex:
'«trk»
'  «name»FMO_Berlin«/name»
'«/trk»
'Returns the "FMO_Berlin" of string.
Public Sub Get_name_trk As String
	If gpx.Length = 0 Then Return ""
	
	Dim str As String = MidExtract(gpx, "<trk>","</trk>")
	str = MidExtract(str, "<name>", "</name>")
	Return str
End Sub
'Ex:
'«trk»
'  «cmt»&lt;a href="#" target="_blank"&gt;Klick&lt;/a&gt;&lt;br /&gt; Ein Link.«/cmt»
'«/trk»
'Returns the "cmt" of string.
Public Sub Get_cmt_trk As String
	If gpx.Length = 0 Then Return ""
	
	Dim str As String = MidExtract(gpx, "<trk>","</trk>")
	str = MidExtract(str, "<cmt>", "</cmt>")
	Return str
End Sub
'Ex:
'«trk»
'	«extensions»
'     «gpxx:TrackExtension xmlns:gpxx="http://www.garmin.com/xmlschemas/GpxExtensions/v3"»
'   «/extensions»
'«/trk»
'Returns the "http://www.garmin.com/xmlschemas/GpxExtensions/v3" of string.
Public Sub Get_TrackExtension_extensions As String
	If gpx.Length = 0 Then Return ""

	Dim str As String = MidExtract(gpx, "<trk>","</trk>")
	str = MidExtract(str, "<gpxx:TrackExtension xmlns:gpxx="&Chr(34), Chr(34)&">")
	Return str
End Sub
'Ex:
'«trk»
'	«extensions»
'     «gpxx:DisplayColor»Blue«/gpxx:DisplayColor»
'	«extensions»
'«/trk»
'Returns the "Blue" of string.
Public Sub Get_DisplayColor_extensions As String
	If gpx.Length = 0 Then Return ""

	Dim str As String = MidExtract(gpx, "<trk>","</trk>")
	str = MidExtract(str, "<gpxx:DisplayColor>", "</gpxx:DisplayColor>")
	Return str
End Sub

'«wpt lat="51.6182327" lon="7.3255205"»
'  «ele»47.722534199999998«/ele»
'  «time»2007-03-31T18:51:17Z«/time»
'  «name»Altes Schiffshebewerk Henrichenburg«/name»
'  «sym»Boat Ramp«/sym»
'  «extensions»
'    «gpxx:WaypointExtension xmlns:gpxx="http://www.garmin.com/xmlschemas/GpxExtensions/v3"»
'      «gpxx:DisplayMode»SymbolAndName«/gpxx:DisplayMode»
'    «/gpxx:WaypointExtension»
'  «/extensions»
'«/wpt»
'Returns the "wpt" of string.
'In this example the function will Return value: lat,lon,ele,time,name,cmt,desc,sym,WaypointExtension,DisplayMode.
'Pattern: splits the given text around matches of the pattern (default: "∆").
Public Sub Get_wpt(Pattern As String) As List
	Dim sta_index, end_index, sum_wpt As List
	sta_index.Initialize 'SearchFor: <wpt>
	end_index.Initialize 'SearchFor: </wpt>
	sum_wpt.Initialize

	If gpx.Length = 0 Then Return sum_wpt
	
	Dim idx As Int = 0
	Dim sum As String = ""
	Dim str As String = gpx

	'SearchFor: "<wpt>"
	Do While idx > -1
		idx = str.IndexOf2("<wpt",idx)   'Returns -1 if SearchFor was not found. ----------- S T A R T.
		If idx > -1 Then
			 sta_index.Add(idx)
			 idx = idx + 4
		End If
	Loop

	idx = 0

	'SearchFor: "</wpt>"
	Do While idx > -1
		idx = str.IndexOf2("</wpt>",idx) 'Returns -1 if SearchFor was not found. ----------- E N D.
		If idx > -1 Then
			 end_index.Add(idx)
			 idx = idx + 6
		End If
	Loop

	If sta_index.Size <> end_index.Size Then
		Return sum_wpt
	Else
		Dim lat,lon,ele,hms,nme,cmt,des,sym,wpe,dmo As String = ""
		If Pattern.Length = 0 Then Pattern = "∆"
		For i = 0 To sta_index.Size - 1
			sum = str.SubString2(sta_index.Get(i), end_index.Get(i))
			lat = MidExtract(sum, "<wpt lat="&Chr(34), Chr(34)&" ")
			lon = MidExtract(sum, "lon="&Chr(34), Chr(34)&">")
			ele = MidExtract(sum, "<ele>", "</ele>")
			hms = MidExtract(sum, "<time>", "</time>")
			hms = MidExtract(hms, "", "T") &" "& MidExtract(hms, "T", "Z")
			nme = MidExtract(sum, "<name>", "</name>")
			cmt = MidExtract(sum, "<cmt>", "</cmt>")
			des = MidExtract(sum, "<desc>", "</desc>")
			sym = MidExtract(sum, "<sym>", "</sym>")
			wpe = MidExtract(sum, "<extensions>", "</extensions>")
			wpe = MidExtract(wpe, "<gpxx:WaypointExtension xmlns:gpxx="&Chr(34), Chr(34)&">")
			dmo = MidExtract(sum, "<gpxx:DisplayMode>", "</gpxx:DisplayMode>")
			sum_wpt.Add(lat &Pattern& lon &Pattern& ele &Pattern& hms &Pattern& nme &Pattern& cmt &Pattern& des &Pattern& sym &Pattern& wpe &Pattern& dmo)
		Next
		Return sum_wpt
	End If
End Sub

'«trkseg»
'  «trkpt lat="52.131124902516603" lon="7.690135566517711"»
'	 «ele»47.722534199999998«/ele»
'    «time»2012-01-27T10:05:02Z«/time»
'  «/trkpt»
'«/trkseg»
'Returns the "trkpt" of string.
'In this example the function will Return value: lat,lon,ele,time.
'Pattern: splits the given text around matches of the pattern (default: "∆").
Public Sub Get_trkpt_trkseg(Pattern As String) As List
	Dim sta_index, end_index, sum_track As List
	sta_index.Initialize 'SearchFor: <trkpt>
	end_index.Initialize 'SearchFor: </trkpt>
	sum_track.Initialize

	If gpx.Length = 0 Then Return sum_track
	
	Dim idx As Int = 0
	Dim sum As String = ""
	Dim str As String = MidExtract(gpx, "<trkseg>","</trkseg>")

	'SearchFor: "<trkpt>"
	Do While idx > -1
		idx = str.IndexOf2("<trkpt",idx)   'Returns -1 if SearchFor was not found. ----------- S T A R T.
		If idx > -1 Then
			 sta_index.Add(idx)
			 idx = idx + 6
		End If
	Loop

	idx = 0

	'SearchFor: "</trkpt>"
	Do While idx > -1
		idx = str.IndexOf2("</trkpt>",idx) 'Returns -1 if SearchFor was not found. ----------- E N D.
		If idx > -1 Then
			 end_index.Add(idx)
			 idx = idx + 8
		End If
	Loop
	
	If sta_index.Size <> end_index.Size Then
		Return sum_track
	Else
		Dim lat,lon,ele,hms As String = ""
		If Pattern.Length = 0 Then Pattern = "∆"
		For i = 0 To sta_index.Size - 1
			sum = str.SubString2(sta_index.Get(i), end_index.Get(i))
			lat = MidExtract(sum, "<trkpt lat="&Chr(34), Chr(34)&" ")
			lon = MidExtract(sum, "lon="&Chr(34), Chr(34)&">")
			ele = MidExtract(sum, "<ele>", "</ele>")
			hms = MidExtract(sum, "<time>", "</time>")
			sum = MidExtract(hms, "", "T") &" "& MidExtract(hms, "T", "Z")
			sum_track.Add(lat &Pattern& lon &Pattern& ele &Pattern& sum)
		Next
		Return sum_track
	End If
End Sub